/*******************************************************************************
 *                               lightconfig.h                                 *
 *              =---------------------------------------------=                *
 *                begin       : Jan 06 2006                                    *
 *                author      : Jun Oyama (aka Metrocrusader)  (C) 2006        *
 *                email       : metrocrusader@mail.com                         *
 *******************************************************************************/

/*
 *      Copyright  :     See COPYING file that comes with this distribution
 */

#include <kconfig.h>
#include <klocale.h>
#include <kglobal.h>
#include <kcolorbutton.h>
#include <kurlrequester.h>


#include <qbuttongroup.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qlabel.h>

#include "lightconfig.h"
#include "configdialog.h"
#include "lightconfig.moc"

const int ButtonTypeCount = 9;



extern "C"
{
  KDE_EXPORT QObject *allocate_config( KConfig* config, QWidget* parent ) {
    return ( new LightConfig( config, parent ) );
  }
}


LightConfig::LightConfig(KConfig *config,QWidget *parent)
  : QObject(parent), config_(0), dialog_(0)
{

  config_ = new KConfig("kwinlightrc");
  KGlobal::locale()->insertCatalogue("kwin_light_config");

  dialog_ = new ConfigDialog(parent);
  dialog_->show();

  load(config_);



  ///////////////////////////////////////////
  //         "General" tab settings
  connect(dialog_->titlealign,SIGNAL(clicked(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->titlesize,SIGNAL(valueChanged(int)),
	  SLOT(selectionChanged(int)));

  connect(dialog_->framesize,SIGNAL(valueChanged(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->framelocation,SIGNAL(clicked(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->gradienttype,SIGNAL(highlighted(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->roundcorner,SIGNAL(stateChanged(int)),
	  SLOT(selectionChanged(int)));
  
  
  ///////////////////////////////////////////
  //         "Buttons" tab settings
  connect(dialog_->buttontype,SIGNAL(highlighted(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->defaultbright,SIGNAL(highlighted(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->brightness,SIGNAL(valueChanged(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->timerinterval,SIGNAL(valueChanged(int)),
	  SLOT(selectionChanged(int)));

  connect(dialog_->spacing,SIGNAL(valueChanged(int)),
	  SLOT(selectionChanged(int)));
  
  connect(dialog_->iscustombutton,SIGNAL(stateChanged(int)),
	  SLOT(selectionChanged(int)));


  connect(dialog_->filepath,SIGNAL(textChanged(const QString &)),
	  SLOT(textChanged(const QString &)));
  connect(dialog_->filepath,SIGNAL(urlSelected(const QString &)),
	  SLOT(textChanged(const QString &)));
  connect(dialog_->filepath,SIGNAL(returnPressed()),
	  SIGNAL(changed()));



  ///////////////////////////////////////////
  //          Color buttons
  connect(dialog_->colorclose,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colormaximize,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorminimumize,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colormenu,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorhelp,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorsticky,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorshade,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorkeepabove,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  connect(dialog_->colorkeepbelow,SIGNAL(changed(const QColor &)),
	  SLOT(colorChanged(const QColor &)));
  
}



LightConfig::~LightConfig()
{
  if(dialog_) delete dialog_;
  if(config_) delete config_;
}


void LightConfig::selectionChanged(int)
{
  emit changed();
}

void LightConfig::textChanged(const QString &)
{
  emit changed();
}

void LightConfig::colorChanged(const QColor &)
{
  emit changed();
}


void LightConfig::load(KConfig *)
{
  int item = 0;
  bool flag = false;
  QString value;
  QString buf;

  
  config_->setGroup("General");

  QRadioButton *button = 0;
  value = config_->readEntry("TitleAlignment","AlignHCenter");
  if(value == "AlignHCenter")    button = dialog_->buttoncenter;
  else if(value == "AlignLeft")  button = dialog_->buttonleft;
  else if(value == "AlignRight") button = dialog_->buttonright;
  if(button) {
    button->setChecked(true);
  }

  dialog_->titlesize->setValue(config_->readNumEntry("TitleSize",20));
  dialog_->framesize->setValue(config_->readNumEntry("FrameSize",3));
  uchar frameflag = (uchar)config_->readNumEntry("FrameLocation",0x07);
  if(frameflag & 0x04) dialog_->frameleft->setChecked(true);
  if(frameflag & 0x02) dialog_->frameright->setChecked(true);
  if(frameflag & 0x01) dialog_->framebottom->setChecked(true);
  
  dialog_->roundcorner->setChecked(config_->readBoolEntry("RoundCorner",true));
  
  dialog_->gradienttype->setCurrentItem(config_->readNumEntry("GradientType",0));
  
  
  dialog_->brightness->setValue(config_->readNumEntry("Brightness",20));
  dialog_->timerinterval->setValue(config_->readNumEntry("TimerInterval",60));
  dialog_->spacing->setValue(config_->readNumEntry("Spacing",6));  

  flag = config_->readBoolEntry("DefaultBright",false);
  if(flag) dialog_->defaultbright->setCurrentItem(0);
  else     dialog_->defaultbright->setCurrentItem(1);
  
  dialog_->iscustombutton->setChecked(config_->readBoolEntry("IsCustomButton",false));

  value = config_->readEntry("ButtonType","Button1");
  if(value == "Button1")            item = 0;
  else if(value == "Button2")       item = 1;
  else if(value == "Button3")       item = 2;      
  else if(value == "Button4")       item = 3;
  dialog_->buttontype->setCurrentItem(item);
    
  dialog_->filepath->setURL(config_->readEntry("CustomButton",""));
  

  QColor color;
  KColorButton *kcbutton[ButtonTypeCount] = {
    dialog_->colorhelp,
    dialog_->colormaximize,
    dialog_->colorminimumize,
    dialog_->colorclose,
    dialog_->colormenu,
    dialog_->colorsticky,
    dialog_->colorkeepabove,
    dialog_->colorkeepbelow,
    dialog_->colorshade
  };
  bool converted = false;
  for(int i = 0;i < ButtonTypeCount; i++) {
    value = "ButtonColor" + QString::number(i);
    buf   = config_->readEntry(value,"0xFFFFFFFF");    // hex
    kcbutton[i]->setColor(QColor(buf.toULong(&converted,16)));
    if(!converted) { // error in hex => ulong conversion
      kcbutton[i]->setColor(white);
    }
  }
}
 


void LightConfig::save(KConfig *)
{
  int item = 0;
  QString value;
  QString buf;

  config_->setGroup("General");


  QRadioButton *button = (QRadioButton *)dialog_->titlealign->selected();
  if(button) {
    if(button == dialog_->buttonleft)        value = "AlignLeft";
    else if(button == dialog_->buttonright)  value = "AlignRight";
    else if(button == dialog_->buttoncenter) value = "AlignHCenter";
    config_->writeEntry("TitleAlignment",value);
  }

  config_->writeEntry("TitleSize",dialog_->titlesize->value());
  config_->writeEntry("FrameSize",dialog_->framesize->value());
  
  uchar frameflag = 0x00;
  frameflag = (dialog_->frameleft->isChecked() << 2) |
    (dialog_->frameright->isChecked() << 1) | (dialog_->framebottom->isChecked());
  config_->writeEntry("FrameLocation",frameflag);
  
  config_->writeEntry("RoundCorner",dialog_->roundcorner->isChecked());
  
  config_->writeEntry("GradientType",dialog_->gradienttype->currentItem());
  

  config_->writeEntry("Brightness",dialog_->brightness->value());
  config_->writeEntry("TimerInterval",dialog_->timerinterval->value());
  config_->writeEntry("Spacing",dialog_->spacing->value());
  
  item = dialog_->defaultbright->currentItem();
  if(item == 0) config_->writeEntry("DefaultBright",true);     // Bright
  else          config_->writeEntry("DefaultBright",false);    // Dark

  config_->writeEntry("IsCustomButton",dialog_->iscustombutton->isChecked());
  
  item = dialog_->buttontype->currentItem();
  value = "Button" + QString::number(item + 1);
  config_->writeEntry("ButtonType",value);
  

  config_->writeEntry("CustomButton",dialog_->filepath->url());
  


  QColor color;
  KColorButton *kcbutton[ButtonTypeCount] = {
    dialog_->colorhelp,
    dialog_->colormaximize,
    dialog_->colorminimumize,
    dialog_->colorclose,
    dialog_->colormenu,
    dialog_->colorsticky,
    dialog_->colorkeepabove,
    dialog_->colorkeepbelow,
    dialog_->colorshade
  };
  for(int i = 0; i<ButtonTypeCount;i++) {
    value = "ButtonColor" + QString::number(i);                 //    hex
    buf   = "0x" + QString::number((uint)(kcbutton[i]->color().rgb()),16);
    config_->writeEntry(value,buf);
  }

  
  config_->sync();
}

void LightConfig::defaults()
{


  QRadioButton *button = (QRadioButton *)dialog_->buttoncenter;
  if(button)  button->setChecked(true);
  
  dialog_->titlesize->setValue(20);
  dialog_->framesize->setValue(3);
  
  dialog_->frameleft->setChecked(true);
  dialog_->frameright->setChecked(true);
  dialog_->framebottom->setChecked(true);
  
  dialog_->roundcorner->setChecked(true);

  dialog_->gradienttype->setCurrentItem(0);
  

  dialog_->brightness->setValue(20);
  dialog_->timerinterval->setValue(60);
  dialog_->spacing->setValue(6);
  
  dialog_->defaultbright->setCurrentItem(0);
  
  dialog_->iscustombutton->setChecked(false);
  
  dialog_->buttontype->setCurrentItem(0);

  dialog_->filepath->setURL("");

  QColor color;
  KColorButton *kcbutton[ButtonTypeCount] = {
    dialog_->colorhelp,
    dialog_->colormaximize,
    dialog_->colorminimumize,
    dialog_->colorclose,
    dialog_->colormenu,
    dialog_->colorsticky,
    dialog_->colorkeepabove,
    dialog_->colorkeepbelow,
    dialog_->colorshade
  };
  for(int i = 0; i<ButtonTypeCount;i++) {
    kcbutton[i]->setColor(QColor(white));
  }
    
}







