/*******************************************************************************
 *                               lightclient.cc                                *
 *              =---------------------------------------------=                *
 *                begin       : Jan 06 2006                                    *
 *                author      : Jun Oyama (aka Metrocrusader)  (C) 2006        *
 *                email       : metrocrusader@mail.com                         *
 *******************************************************************************/

/*                                                                             
 *      Copyright  :     See COPYING file that comes with this distribution
 */


#include <kconfig.h>
#include <kdeversion.h>
#include <kdrawutil.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>


#include <qfile.h>
#include <qimage.h>
#include <qtimer.h>
#include <qstring.h>
#include <qbitmap.h>
#include <qdrawutil.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtooltip.h>

#include "lightclient.h"

// button resources
#include "btns.h" 



using namespace Light;
bool LightFactory::flag_initialized          = false;

////////////////////////////////////////////////////////////
//            Default Values

QPixmap *LightFactory::pixmapCacheTable_[]   = {0};
QImage  *LightFactory::buttonImageCache_     = 0;


Qt::AlignmentFlags LightFactory::titleAlign_ = Qt::AlignHCenter;
int LightFactory::timerInterval_             = 60;
int LightFactory::titleSize_                 = 21;
int LightFactory::frameSize_                 = 3;
uchar LightFactory::frameLocationFlag_       = 0x07; // left | right | bottom
int LightFactory::gradientType_              = 0;
int LightFactory::brightness_                = 20;   // may be used in the future
int LightFactory::Spacing_                   = 6;
QRgb LightFactory::buttonColor_[]            = {0};
bool LightFactory::defaultBright_            = false;
bool LightFactory::roundCorner_              = true;


////////////////////////////////////////////////////////////
//          Global hook point

extern "C" KDecorationFactory *create_factory()
{
  return new Light::LightFactory();
}



// class LightFactory : public KDecorationFactory
// {

// public:
//   LightFactory();
//   virtual ~LightFactory();
//   virtual KDecoration *createDecoration(KDecorationBridge *b);
//   virtual bool reset(unsigned long changed);

//   static bool initialized();

//   static Qt::AlignmentFlags titleAlign();
//   static int timerInterval();
//   static int titleSize();
//   static int frameSize();
//   static uchar frameLocationFlag();
//   static int gradientType();
//   static int brightness();
//   static int Spacing();
//   static QRgb buttonColor(int);
//   static bool defaultBright();
//   static bool roundCorner();

//   static QPixmap *pixmapCacheTable(int);
//   static QImage *buttonImageCache();
  
//   void createAlphaMask(QImage *,const QImage *, const int Alpha) const;


// private:
//   bool readConfig();
  
// private:
//   static bool flag_initialized;
//   static QPixmap *pixmapCacheTable_[lightEffect_steps];
//   static QImage *buttonImageCache_;

//   // configuration stuff
//   static Qt::AlignmentFlags titleAlign_;
//   static int timerInterval_;
//   static int titleSize_;
//   static int frameSize_;
//   static uchar frameLocationFlag_; 
//   static int gradientType_;
//   static int brightness_;
//   static int Spacing_;
//   static QRgb buttonColor_[ButtonTypeCount];
//   static bool defaultBright_;
//   static bool roundCorner_;
  
// };


////////////////////////////////////////////////////////////
// LightFactory
//=========================================================/
// constructor
LightFactory::LightFactory()
{
  // initialize the factory.
  for(int i = 0;i<lightEffect_steps;i++) pixmapCacheTable_[i] = new QPixmap;
  buttonImageCache_ = new QImage;
 
  readConfig();

  flag_initialized = true;
}



////////////////////////////////////////////////////////////
// LightFactory destructor
LightFactory::~LightFactory()
{
  
  for(int i = 0;i<lightEffect_steps;i++) 
    if(pixmapCacheTable_[i]) delete pixmapCacheTable_[i];
  if(buttonImageCache_)      delete buttonImageCache_;
  
  flag_initialized = false;
}


////////////////////////////////////////////////////////////
// create each Decoration
KDecoration *LightFactory::createDecoration(KDecorationBridge *b)
{
  return new LightClient(b,this);
}


////////////////////////////////////////////////////////////
// reset Decorations
bool LightFactory::reset(unsigned long changed)
{
  // delete all the cache
  for(int i = 0;i<lightEffect_steps;i++) 
    if(pixmapCacheTable_[i]) delete pixmapCacheTable_[i];
  if(buttonImageCache_)      delete buttonImageCache_;
    
 
  // reallocate the stuff
  for(int i = 0;i<lightEffect_steps;i++) pixmapCacheTable_[i] = new QPixmap;
  buttonImageCache_ = new QImage;
  
  flag_initialized = false;
  bool confchange = readConfig();
  flag_initialized = true;
  
  if(confchange || (changed & (SettingDecoration | SettingButtons | SettingBorder))) {
    return true;
  }
  else {
    resetDecorations(changed);
    return false;
  }
}

////////////////////////////////////////////////////////////
// read configuration stuff
bool LightFactory::readConfig()
{
  QString value;
  QString buf;
  QImage tmp;
  char **buttonType = 0; // xpm
  

  // create config object
  KConfig config("kwinlightrc");
  config.setGroup("General");
  
  //read settings
  value  = config.readEntry("TitleAlignment","AlignHCenter");
  if(value == "AlignLeft")         titleAlign_ = Qt::AlignLeft;
  else if(value == "AlignHCenter") titleAlign_ = Qt::AlignHCenter;
  else if(value == "AlignRight")   titleAlign_ = Qt::AlignRight; 
  
  timerInterval_ = config.readNumEntry("TimerInterval",60);
  titleSize_     = config.readNumEntry("TitleSize",20);
  frameSize_     = config.readNumEntry("FrameSize",3);
  frameLocationFlag_ = (uchar)config.readNumEntry("FrameLocation",0x07);
  
  gradientType_  = config.readNumEntry("GradientType",0);
  
  
  bool isCustomButton = config.readBoolEntry("IsCustomButton","false");
  if(isCustomButton == true) {
    value = config.readEntry("CustomButton","");
    if(value.isEmpty() || !QFile::exists(value)) {
      value = "";
      buttonType = btn1_xpm; // set normal button
      tmp = QImage(buttonType);
    }
    else {
      tmp = QImage(value);
      if(tmp.isNull()) {
 	buttonType = btn1_xpm; // sets the normal button when invalid image file
 	tmp = QImage(buttonType);
      }
    }
  }
  else {      // when we do not want to use custom button image
    value  = config.readEntry("ButtonType","Normal");
    if(value == "Button1")            buttonType = btn1_xpm;
    else if(value == "Button2")      buttonType = btn2_xpm;
    else if(value == "Button3")      buttonType = btn3_xpm;
    else if(value == "Button4")      buttonType = btn4_xpm;
    else                             buttonType = btn1_xpm;
    tmp = QImage(buttonType);
  }
  tmp = tmp.convertDepth(32);
  *buttonImageCache_ = tmp.copy(); // sets the image to cache
  

  
  brightness_    = config.readNumEntry("Brightness",20);
  Spacing_       = config.readNumEntry("Spacing",6);
  
  bool converted;
  for(int i = 0;i<ButtonTypeCount; i++) {
    value = "ButtonColor" + QString::number(i);
    buf = config.readEntry(value,"0");
    buttonColor_[i] = (QRgb)(buf.toULong(&converted,16));  // uint hex
    if(!converted) { // conversion error occured
      buttonColor_[i] = 0xFFFFFFFF; // default color
    }
  }
  

  
  defaultBright_ = config.readBoolEntry("DefaultBright",false);
  roundCorner_   = config.readBoolEntry("RoundCorner",true);
  
  if(timerInterval_ < 0) timerInterval_ = 0;
  
  if(brightness_ < 0) brightness_ = 0;
  else if(brightness_ > 100) brightness_ = 100;
  
  if(Spacing_ < 0) Spacing_ = 0;
  

  //
  //  Creates the pixmap cache for quick drawing
  // 
  if(pixmapCacheTable_[0]->isNull()) {

    int Alpha_first = 0xFF * (100-brightness_) / 100;
    signed int Alpha_delta = Alpha_first / lightEffect_steps;
    
    if(defaultBright_) { // when bright -> dark
      Alpha_first = 0;
      Alpha_delta *= (-1);
    }
    
    for(int i = 0 ; i<lightEffect_steps ; i++) {
      
      if(Alpha_first != 0 && i == lightEffect_steps - 1) // lighting && the last one
	createAlphaMask(&tmp,buttonImageCache_,0);       // the brightest one's alpha 
                                                         // should be Zero.
      else createAlphaMask(&tmp,buttonImageCache_,Alpha_first - i*Alpha_delta);
      
      pixmapCacheTable_[i]->convertFromImage(tmp);
    }
    
  }
  

  return true;

}


////////////////////////////////////////////////////////////
// make Alpha pixmaps cache for quick drawing
// this needed to be done right after it reads the configuration
void LightFactory::createAlphaMask(QImage *newimage,const QImage *original, const int Alpha) const
{

  //  This function creates the one color (black) image, which has a 
  //  specified Alpha, so that it can be painted over the button picture 
  //  to make it have light effect.
  //
  int h = original->height(), w = original->width(); 
  
  QRgb *srcline,*dstline;  
  *newimage = original->copy();         // DEEP COPY
  
  int a;
  for(int i = 0; i<h ; i++) {
    srcline = (QRgb *)original->scanLine(i);
    dstline = (QRgb *)newimage->scanLine(i);
    for(int j = 0; j<w; j++) {

      a = qAlpha(*(srcline+j)) ? Alpha : 0 ;

      *(dstline+j) = qRgba(0,0,0,a);
    }
  }
}






// class LightButton : public QButton
// {
//   Q_OBJECT

// public:
//   LightButton(LightClient *parent = 0, const char *name = 0,
// 	      const QString &tip=NULL,
// 	      ButtonType buttontype=ButtonHelp);
//   ~LightButton();
  

//   QSize sizeHint() const; 
//   ButtonState lastMousePress() const;
  
// public slots:
//   void reset();              // because parent's redraw forces button redraw also.
//   void timerDone(); 

// protected:
//   void colorizeButton(QImage *,const QImage *, const QRgb) const; 



//   void paintEvent(QPaintEvent *);
//   void enterEvent(QEvent *);
//   void leaveEvent(QEvent *);
//   void mousePressEvent(QMouseEvent *);
//   void mouseReleaseEvent(QMouseEvent *);
  
// private:
//   QSize       size;
//   QPixmap     *pixmap_btn;
//   LightClient *client;       // its parent
//   ButtonType  type;
//   ButtonState lastmouse;
//   bool        flag_insideButton;
//   int         currentStep;
//   QTimer      *lightTimer;
// };


////////////////////////////////////////////////////////////
// LightButton
//=========================================================/
// constructor
LightButton::LightButton(LightClient *parent,const char *name,
			 const QString &tip,ButtonType buttontype)
  : QButton(parent->widget(),name),
    pixmap_btn(0),
    client(parent), type(buttontype),lastmouse(NoButton),
    flag_insideButton(false),currentStep(0),lightTimer(0)
{
  setBackgroundMode(NoBackground);
  setCursor(arrowCursor);
  
  lightTimer = new QTimer(this);
  connect(lightTimer,SIGNAL(timeout()),this,SLOT(timerDone()));
  
  
  QImage *image = LightFactory::buttonImageCache();

  size = image->size();     // sets the size

  setFixedSize(size);       // fixed size
  
  //   ///////////////////////////////////////////////
  //   // create color button Image
  QImage tmp;

  QRgb rgb = LightFactory::buttonColor(type);
  colorizeButton(&tmp,image,rgb);
  pixmap_btn = new QPixmap(size);
  pixmap_btn->convertFromImage(tmp);
  //   ///////////////////////////////////////////////
  

  QToolTip::add(this,tip);
}


////////////////////////////////////////////////////////////
// destructor
LightButton::~LightButton()
{
  if(lightTimer) delete lightTimer;
  if(pixmap_btn) delete pixmap_btn;
}



////////////////////////////////////////////////////////////
// set colorful buttons
void LightButton::colorizeButton(QImage *newimage,const QImage *original, const QRgb color) const
{
  int h = original->height(), w = original->width(); 
  // ratio for each color
  double rr = (double)qRed(color)/0xFF;
  double gr = (double)qGreen(color)/0xFF;
  double br = (double)qBlue(color)/0xFF;

  int r,g,b,a;

  QRgb *srcline,*dstline;

  *newimage = original->copy();          // DEEP COPY
  
  for(int i = 0; i<h ; i++) {
    srcline = (QRgb *)original->scanLine(i);
    dstline = (QRgb *)newimage->scanLine(i);
    for(int j = 0; j<w; j++) {
      r = qRed(*(srcline+j));
      g = qGreen(*(srcline+j));
      b = qBlue(*(srcline+j));
      a = qAlpha(*(srcline+j)) ? qAlpha(color) : 0; 
      
      if(r <= 0x70 && b <= 0x70 && g <= 0x70 && a != 0 )
 	a = 0x80;  // dark pixels transculency to match background.
      
      *(dstline + j) = qRgba((int)(r*rr),(int)(g*gr),(int)(b*br),a);
      
    }
  }
}




////////////////////////////////////////////////////////////
// sizeHint
QSize LightButton::sizeHint() const
{
  return size;
}



////////////////////////////////////////////////////////////
// done at timer Intrupt function
void LightButton::timerDone()
{
  if(flag_insideButton == true && currentStep < lightEffect_steps - 1) 
    currentStep++;
  
  else if(flag_insideButton == false && currentStep > 0)
    currentStep--;
  
  if(currentStep <=0 || lightEffect_steps -1 <= currentStep)
    lightTimer->stop();
  reset();
}



////////////////////////////////////////////////////////////
// do the "mouseover effect"
void LightButton::enterEvent(QEvent *e)
{
  flag_insideButton = true;
  lightTimer->start(LightFactory::timerInterval());     // NOT a singleshot timer!!!
  
  QButton::enterEvent(e);
}



////////////////////////////////////////////////////////////
// do the "mouseover effect"
void LightButton::leaveEvent(QEvent *e)
{
  flag_insideButton = false;
  if(!lightTimer->isActive()) lightTimer->start(LightFactory::timerInterval());
  
  QButton::leaveEvent(e);
}



////////////////////////////////////////////////////////////
// light up the button when clicked & do the Clicking operations
void LightButton::mousePressEvent(QMouseEvent *e)
{
  if(LightFactory::defaultBright()) currentStep = 0;   // always the brightest one
  else   currentStep = lightEffect_steps - 1;
  
  
  lastmouse = e->button();
  
  int button = LeftButton;
  if((type != ButtonMax) && (e->button() != LeftButton)) {
    button = NoButton;
  }
  QMouseEvent me(e->type(),e->pos(),e->globalPos(),button,e->state());
  QButton::mousePressEvent(&me);

  // this code lights up the button when defaultBright is true
  if(!lightTimer->isActive()) lightTimer->start(LightFactory::timerInterval());
}




////////////////////////////////////////////////////////////
// do the Releasing operations
void LightButton::mouseReleaseEvent(QMouseEvent *e)
{
  lastmouse = e->button();
  
  int button = LeftButton;
  if((type != ButtonMax) && (e->button() != LeftButton)) {
    button = NoButton;
  }
  QMouseEvent me(e->type(), e->pos(), e->globalPos(), button,e->state());
  QButton::mouseReleaseEvent(&me);
}





////////////////////////////////////////////////////////////
// make button Image.
void LightButton::paintEvent(QPaintEvent *)
{
  if(!LightFactory::initialized()) return;

  static QPixmap pixmap;
  static QPixmap pixmap_bg;
  
  // resizeing is essential
  pixmap.resize(size);
  pixmap_bg.resize(size);

  // this must be done here because we cannot do it in the constructor
  QPoint buttonPosition = mapToParent(QPoint(0,0)); // for the round frame.
  if(LightFactory::roundCorner()) buttonPosition -= QPoint(1,1);
  //  *****  IMPORTANT  ***** 
  // take the right part from the pixmap background cache in the 
  // parent class and transfer it to the pixmap_background.
  bitBlt(&pixmap_bg,0,0,&(client->titlebar_bg),
	 buttonPosition.x(),buttonPosition.y(),size.width(),size.height());

  
  QPainter painter;
  
  painter.begin(&pixmap);
  
  painter.drawPixmap(0,0,pixmap_bg);       // draw background on pixmap
  
  if(pixmap_btn != 0) {
    painter.drawPixmap(0,0,*pixmap_btn);   // draw button picture on pixmap
  }
  
  if(LightFactory::pixmapCacheTable(currentStep)) {
    painter.drawPixmap(0,0,*LightFactory::pixmapCacheTable(currentStep));
  }
  
  if(! client->isActive()) {              // dark buttons on inactive windows
    painter.drawPixmap(0,0,*LightFactory::pixmapCacheTable(4));
  }
  
  painter.end(); // pixmap

  painter.begin(this);
  painter.drawPixmap(0,0,pixmap);
  painter.end(); // this

}





// class LightClient : public KDecoration
// {
//   Q_OBJECT

// signals:
//   void buttonRepaint();               // connected to each button's reset

// public:
//   LightClient(KDecorationBridge *b,KDecorationFactory *f);
//   virtual ~LightClient();
  
//   virtual void init(); 
//   virtual void borders(int &,int &,int &,int &) const;
//   virtual void resize(const QSize &);
//   virtual QSize minimumSize() const;
  
//   void resizeEvent(QResizeEvent *);
//   void paintEvent(QPaintEvent *);
//   void showEvent(QShowEvent *);
//   void mouseDoubleClickEvent(QMouseEvent *);

//   virtual void maximizeChange();
//   virtual void activeChange();
//   virtual void iconChange();
//   virtual void desktopChange();
//   virtual void shadeChange();
//   virtual void captionChange();

//   void doShape();

//   virtual KDecoration::Position mousePosition(const QPoint &) const;
//   bool eventFilter(QObject *,QEvent *);

//   QPixmap titlebar_bg;                    // for background   
  
// private:
//   void addButtons(QBoxLayout *layout,const QString &buttons);

// private slots:  
//   void maxButtonPressed();
//   void menuButtonPressed();
//   void aboveButtonPressed();  
//   void belowButtonPressed();
//   void shadeButtonPressed();
//   void keepAboveChange(bool);
//   void keepBelowChange(bool);


// private:
//   LightButton *button[ButtonTypeCount];
//   QHBoxLayout *entire_titlebar;
//   QSpacerItem *titlebar;
// };


////////////////////////////////////////////////////////////
// LightClient
//=========================================================/
// constructor
LightClient::LightClient(KDecorationBridge *b,KDecorationFactory *f)
  : KDecoration(b,f)
{
  ;     // do not do anything
}


////////////////////////////////////////////////////////////
// destructor
LightClient::~LightClient()
{
  for(int n=0 ; n<ButtonTypeCount ; n++) {
    if(button[n]) delete button[n];
  }
}



////////////////////////////////////////////////////////////
// the REAL window initializer
void LightClient::init()
{

//                 mainlayout
//                     \/
//        +---------------------------+
//        |oo|       caption      |ooo|    <- entire_titlebar
//        +------------------------+--+ 
//        | |                       | |  
//        | |                       | |
//        | |                       | |
//        | |                       | |
//        | |                       | |
//        | |      contents         | |    <- mainwindow
//        | |                       | |
//        | |                       | |
//        | |                       | |
//        | |                       | |
//        | |                       | |
//        +-+-----------------------+-+
//        +---------------------------+



  createMainWidget(WResizeNoErase | WRepaintNoErase);
  widget()->installEventFilter(this);
  
  widget()->setBackgroundMode(NoBackground);
  
  QVBoxLayout *mainlayout = new QVBoxLayout(widget()); // entire window
  entire_titlebar = new QHBoxLayout();                 // titlebar
  QHBoxLayout *mainwindow = new QHBoxLayout();         // main part 

  titlebar = new QSpacerItem(1,LightFactory::titleSize(),QSizePolicy::Expanding,QSizePolicy::Fixed);
  
  mainlayout->setResizeMode(QLayout::FreeResize);
  mainlayout->addLayout(entire_titlebar);

  for(int n=0;n<ButtonTypeCount; n++) button[n] = 0;
  entire_titlebar->addSpacing(3);
  addButtons(entire_titlebar,options()->titleButtonsLeft());
  entire_titlebar->addItem(titlebar);
  addButtons(entire_titlebar,options()->titleButtonsRight());
  entire_titlebar->addSpacing(3);

  
  if(isPreview()) {
    mainwindow->addWidget(new QLabel(i18n("<b><center>Light Preview</center></b>"),
				     widget()));
  } else {
    mainwindow->addItem(new QSpacerItem(0,0));   // contents
  }
  // spacing at the right
  mainwindow->addSpacing(LightFactory::frameSize());
  mainlayout->addLayout(mainwindow);
  
  // spacing at the bottom
  mainlayout->addSpacing(LightFactory::frameSize());

  
  connect(this,SIGNAL(keepAboveChanged(bool)),SLOT(keepAboveChange(bool)));
  connect(this,SIGNAL(keepBelowChanged(bool)),SLOT(keepBelowChange(bool)));
  

  for(int n = 0; n < ButtonTypeCount; n++)
    if(button[n])
      connect(this,SIGNAL(buttonRepaint()),button[n],SLOT(reset()));
}



////////////////////////////////////////////////////////////
// add buttons
void LightClient::addButtons(QBoxLayout *layout, const QString& s)
{
  QString tip;

  if (s.length() > 0) {
    for (unsigned n=0; n < s.length(); n++) {
      switch (s[n]) {
      case 'M': // Menu button
	if (!button[ButtonMenu]) {
	  button[ButtonMenu] =
	    new LightButton(this, "menu", i18n("Menu"),
			    ButtonMenu);
	  connect(button[ButtonMenu], SIGNAL(pressed()),
		  this, SLOT(menuButtonPressed()));
	  // Little spacing between each buttons
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonMenu]);
	}
	break;
	
      case 'S': // Sticky button
	if (!button[ButtonSticky]) {
	  if (isOnAllDesktops()) {
	    tip = i18n("Un-Sticky");
	  } else {
	    tip = i18n("Sticky");
	  }
	  button[ButtonSticky] =
	    new LightButton(this, "sticky", tip,
			    ButtonSticky);
	  connect(button[ButtonSticky], SIGNAL(clicked()),
		  this, SLOT(toggleOnAllDesktops()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonSticky]);
	}
	break;
	
      case 'H': // Help button
	if ((!button[ButtonHelp]) && providesContextHelp()) {
	  button[ButtonHelp] =
	    new LightButton(this, "help", i18n("Help"),
			    ButtonHelp);
	  connect(button[ButtonHelp], SIGNAL(clicked()),
		  this, SLOT(showContextHelp()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonHelp]);
	}
	break;
	
      case 'I': // Minimize button
	if ((!button[ButtonMin]) && isMinimizable())  {
	  button[ButtonMin] =
	    new LightButton(this, "iconify", i18n("Minimize"),
			    ButtonMin);
	  connect(button[ButtonMin], SIGNAL(clicked()),
		  this, SLOT(minimize()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonMin]);
	}
	break;
	
      case 'A': // Maximize button
	if ((!button[ButtonMax]) && isMaximizable()) {
	  if (maximizeMode() == MaximizeFull) {
	    tip = i18n("Restore");
	  } else {
	    tip = i18n("Maximize");
	  }
	  button[ButtonMax]  =
	    new LightButton(this, "maximize", tip,
			    ButtonMax);
	  connect(button[ButtonMax], SIGNAL(clicked()),
		  this, SLOT(maxButtonPressed()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonMax]);
	}
	break;
	
      case 'X': // Close button
	if ((!button[ButtonClose]) && isCloseable()) {
	  button[ButtonClose] =
	    new LightButton(this, "close", i18n("Close"),
			    ButtonClose);
	  connect(button[ButtonClose], SIGNAL(clicked()),
		  this, SLOT(closeWindow()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonClose]);
	}
	break;
	
      case 'F': // Above button
	if ((!button[ButtonAbove])) {
	  button[ButtonAbove] =
	    new LightButton(this, "above",
			    i18n("Keep Above Others"), ButtonAbove);
	  connect(button[ButtonAbove], SIGNAL(clicked()),
		  this, SLOT(aboveButtonPressed()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonAbove]);
	}
	break;
	  
      case 'B': // Below button
	if ((!button[ButtonBelow])) {
	  button[ButtonBelow] =
	    new LightButton(this, "below",
			    i18n("Keep Below Others"), ButtonBelow);
	  connect(button[ButtonBelow], SIGNAL(clicked()),
		  this, SLOT(belowButtonPressed()));
	  layout->addSpacing(LightFactory::Spacing());
	  layout->addWidget(button[ButtonBelow]);
	}
	  break;
	  
      case 'L': // Shade button
	if ((!button[ButtonShade && isShadeable()])) {
	  if ( isSetShade()) {
	    tip = i18n("Unshade");
	  } else {
	    tip = i18n("Shade");
	  }
	  button[ButtonShade] =
	    new LightButton(this, "shade", tip,
			    ButtonShade);
	  connect(button[ButtonShade], SIGNAL(clicked()),
		  this, SLOT(shadeButtonPressed()));
	    layout->addSpacing(LightFactory::Spacing());
	    layout->addWidget(button[ButtonShade]);
	}
	break;
	
      case '_': // Spacer item
	layout->addSpacing(LightFactory::Spacing());
      }
    }
  }
  layout->addSpacing(LightFactory::Spacing());
  
}

////////////////////////////////////////////////////////////
// window Active stae change
void LightClient::activeChange()
{
  for(int n=0;n<ButtonTypeCount;n++)
    if(button[n]) button[n]->reset();
  
  widget()->repaint(false);
}

////////////////////////////////////////////////////////////
// caption text change
void LightClient::captionChange()
{
  widget()->repaint(entire_titlebar->geometry(),false); // to draw the entire titlebar ?
}

////////////////////////////////////////////////////////////
// desktop change
void LightClient::desktopChange()
{
  bool d = isOnAllDesktops();
  if(button[ButtonSticky]) {
    QToolTip::remove(button[ButtonSticky]);
    QToolTip::add(button[ButtonSticky], d ? i18n("Un-Sticky") : i18n("Sticky"));
  }
}

////////////////////////////////////////////////////////////
// icon change, but not relevant on this iconless theme
void LightClient::iconChange()
{
}

////////////////////////////////////////////////////////////
// maximized or restored
void LightClient::maximizeChange()
{
  bool m = (maximizeMode() == MaximizeFull);
  
  if(button[ButtonMax]) {
    QToolTip::remove(button[ButtonMax]);
    QToolTip::add(button[ButtonMax], m ? i18n("Restore") : i18n("Maximize"));
  }
}

////////////////////////////////////////////////////////////
// shaded
void LightClient::shadeChange()
{
  bool s = isSetShade();
  if(button[ButtonShade]) {
    QToolTip::remove(button[ButtonShade]);
    QToolTip::add(button[ButtonShade], s ? i18n("Unshade") : i18n("Shade"));
  }
}

////////////////////////////////////////////////////////////
// not relevant as this theme has no icon change
void LightClient::keepAboveChange(bool a)
{
  if(button[ButtonAbove]) {
    QToolTip::remove(button[ButtonAbove]);
    QToolTip::add(button[ButtonAbove], a ? i18n("Do not Keep Above Others") : i18n("Keep Above Others"));
  }
}

////////////////////////////////////////////////////////////
// me neither
void LightClient::keepBelowChange(bool b)
{
  if(button[ButtonBelow]) {
    QToolTip::remove(button[ButtonBelow]);
    QToolTip::add(button[ButtonBelow], b ? i18n("Do not Keep Below Others") : i18n("Keep Below Others"));
  }
}

////////////////////////////////////////////////////////////
// return borders
void LightClient::borders(int &l,int &r,int &t,int &b) const
{
  const uchar frameflag = LightFactory::frameLocationFlag();
  b = (frameflag & 0x01) ? LightFactory::frameSize() : 0;
  r = (frameflag & 0x02) ? LightFactory::frameSize() : 0;
  l = (frameflag & 0x04) ? LightFactory::frameSize() : 0; 
  t = LightFactory::titleSize();
}

////////////////////////////////////////////////////////////
// resized
void LightClient::resize(const QSize &size)
{
  widget()->resize(size);
}

QSize LightClient::minimumSize() const
{
  return widget()->minimumSize();
}


////////////////////////////////////////////////////////////
// returns logical position of cursor
KDecoration::Position LightClient::mousePosition(const QPoint &point) const
{
  const int corner = 24;
  const int framesize = LightFactory::frameSize();
  Position pos;

  if(point.y() <= framesize) {
    // inside top frame
    if(point.x() <= corner)                       pos = PositionTopLeft;
    else if(point.x()>= (width() - corner))       pos = PositionTopRight;
    else                                          pos = PositionTop;
  } else if(point.y() >= (height() - framesize)) {
    // inside bottom frame
    if(point.x() <= corner)                       pos = PositionBottomLeft;
    else if(point.x() >= (width() - corner))      pos = PositionBottomRight;
    else                                          pos = PositionBottom;
  } else if(point.x() <= framesize) {
    // on left frame
    if(point.y() <= corner)                       pos = PositionTopLeft;
    else if(point.y() >= (height() - corner))     pos = PositionBottomLeft;
    else                                          pos = PositionLeft;
  } else if(point.x() >= (width() - framesize)) {
    // on right frame
    if(point.y() <= corner)                       pos = PositionTopRight;
    else if(point.y() >= (height() - corner))     pos = PositionBottomRight;
    else                                          pos = PositionRight;
  } else {
    // inside the frame
    pos = PositionCenter;
  }
  return pos;
}


////////////////////////////////////////////////////////////
// eventFilter fanction
bool LightClient::eventFilter(QObject *obj, QEvent *e)
{
  if (obj != widget()) return false;
  
  switch (e->type()) {
  case QEvent::MouseButtonDblClick: 
    mouseDoubleClickEvent(static_cast<QMouseEvent *>(e));
    return true;
    
  case QEvent::MouseButtonPress:
    processMousePressEvent(static_cast<QMouseEvent *>(e));
    return true;
    
  case QEvent::Paint: 
    paintEvent(static_cast<QPaintEvent *>(e));
    return true;
    
  case QEvent::Resize: 
    resizeEvent(static_cast<QResizeEvent *>(e));
    return true;
    
  case QEvent::Show: 
    showEvent(static_cast<QShowEvent *>(e));
    return true;
    
  default: 
    return false;
    
  }
  
  return false;
}

////////////////////////////////////////////////////////////
// double click
void LightClient::mouseDoubleClickEvent(QMouseEvent *e)
{
  if(titlebar->geometry().contains(e->pos())) titlebarDblClickOperation();
}


////////////////////////////////////////////////////////////
// create the window image
void LightClient::paintEvent(QPaintEvent *)
{
  if(!LightFactory::initialized()) return;
  
  
  QColor titleColor = options()->color(ColorTitleBar,isActive());
  QColor titleBlendColor = options()->color(ColorTitleBlend,isActive());

  QColorGroup group;
  QPainter painter; 

  int gradient = LightFactory::gradientType();

  // draw the titlebar
  QRect title(entire_titlebar->geometry());        
  QRect titletext(titlebar->geometry());            // for caption drawing only
  QSize titlebarSize(title.width(),title.height());

  if(!titlebarSize.isValid()) {
    titlebarSize = QSize(0,0);
  }
  KPixmap gradientPixmap(titlebarSize);
  if(!titlebarSize.isNull() && LightFactory::gradientType() < 7) 
    KPixmapEffect::gradient(gradientPixmap,titleColor,titleBlendColor,
			    (KPixmapEffect::GradientType)gradient);
  
  if(gradient >= 7) {// special stuff
    
    
    switch (gradient) {
      
    case 7:                        // thin stripes, like MacOS
      {
	const int rectheight = 2;
	int recttimes = LightFactory::titleSize() / ( rectheight * 2 ) + 1;
	gradientPixmap.fill(titleColor);
	painter.begin(&gradientPixmap);
	painter.setPen(titleBlendColor);
	painter.setBrush(titleBlendColor);
	for(int i = 0; i< recttimes ; i++) {
	  painter.drawRect(0,i * rectheight * 2,gradientPixmap.width(),rectheight);
	}
	painter.end(); // gradientPixmap
	break;
      }
    case 8:
      {                              // checkerboard. full respect to SKA!
	const int checksize = gradientPixmap.height() / 5;
	const int checkline = gradientPixmap.height() / 10;                 // vertical
	const int checktime = gradientPixmap.width() / (checksize * 2) + 1; // horizontal
	
	gradientPixmap.fill(titleColor);
	painter.begin(&gradientPixmap);
	painter.setPen(titleBlendColor);
	painter.setBrush(titleBlendColor);
	for(int i = 0;i < checkline ; i++) {
	  for(int j = 0;j < checktime ; j++) {
	    painter.drawRect(checksize * (i%2 ? j * 2 : j * 2 + 1),
			     gradientPixmap.height()/2 - checkline / 2 * checksize + i * checksize,
			     checksize,checksize);
	  }
	}
	painter.end(); // gradientPixmap
	break;
      }
    }
  }
    

  // title_buffer is also used as a background cache for each buttons.
  QPixmap title_buffer(titlebarSize);

  painter.begin(&title_buffer);

  if(!gradientPixmap.isNull())
    painter.drawPixmap(0,0,gradientPixmap);
  
  if(titlebarSize.width() >= 3 && titlebarSize.height() >= 3) {
    painter.setFont(options()->font(isActive(),false));
    painter.setPen(options()->color(KDecoration::ColorFont,isActive()));
    painter.drawText(titletext.x(),0,titletext.width(),titletext.height(),
		     LightFactory::titleAlign() | Qt::AlignVCenter | Qt::SingleLine
		     ,caption());

  }
  
  painter.end(); // title_buffer
  if(LightFactory::roundCorner())
    bitBlt(widget(),1,1,&title_buffer);   // 1,1 for the frame drawn by kDrawRoundButton
  else
    bitBlt(widget(),0,0,&title_buffer);

  // for button background cache
  titlebar_bg = title_buffer;
  
  
  painter.begin(widget());
  
  group = options()->colorGroup(KDecoration::ColorFrame,isActive());
  
  // draw the border frames. only right and bottom
  QRect frame;
  const uchar frameflag = LightFactory::frameLocationFlag();
  const int framesize = LightFactory::frameSize();
  
  if(frameflag & 0x01) {
    frame.setRect(0,height() - framesize,width(),framesize); // the bottom one
    painter.fillRect(frame,group.background());
  }  
  if(frameflag & 0x02) {
    frame.setRect(width()-framesize,LightFactory::titleSize(),framesize,height() - LightFactory::titleSize());// right one
    painter.fillRect(frame,group.background());
  }
  if(frameflag & 0x04) {
    frame.setRect(0,LightFactory::titleSize(),framesize,height()-LightFactory::titleSize());//left one
    painter.fillRect(frame,group.background());
  }
  
   // make the window corners round.
  if(LightFactory::roundCorner())
    kDrawRoundButton(&painter,0,0,width(),height(),group,false);
  
  painter.end(); // widget()
  
  // buttons has to be repainted because background changes.
  emit buttonRepaint();

}


////////////////////////////////////////////////////////////
// for round corners
void LightClient::doShape()
{
  QRegion mask;
  kRoundMaskRegion(mask,0,0,width(),height());
  setMask(mask);
}

////////////////////////////////////////////////////////////
// resizing
void LightClient::resizeEvent(QResizeEvent *)
{
  if(LightFactory::roundCorner())
    doShape();
  if(widget()->isShown()) {
    QRegion region = widget()->rect();
    region = region.subtract(titlebar->geometry());
    widget()->erase(region);
  }
}

////////////////////////////////////////////////////////////
// showEvent
void LightClient::showEvent(QShowEvent *)
{
  if(LightFactory::roundCorner())
    doShape();
  widget()->repaint();
}

////////////////////////////////////////////////////////////
// maxButtonPressed
void LightClient::maxButtonPressed()
{
  if(button[ButtonMax]) {
    maximize(button[ButtonMax]->lastMousePress());
  }
}

////////////////////////////////////////////////////////////
// aboveButtonpressed
void LightClient::aboveButtonPressed()
{
  if(button[ButtonAbove]) {
    setKeepAbove(!keepAbove());
  }
}

////////////////////////////////////////////////////////////
// belowButtonPressed
void LightClient::belowButtonPressed()
{
  if(button[ButtonBelow]) {
    setKeepBelow(!keepBelow());
  }
}

////////////////////////////////////////////////////////////
// shadeButtonPressed
void LightClient::shadeButtonPressed()
{
  if(button[ButtonShade]) {
    setShade(!isSetShade());
  }
}

////////////////////////////////////////////////////////////
// menuButtonPressed show the Menu at appropriate position
void LightClient::menuButtonPressed()
{
  if(button[ButtonMenu]) {
    QPoint p(button[ButtonMenu]->rect().bottomLeft().x(),
	     button[ButtonMenu]->rect().bottomLeft().y());
    KDecorationFactory *f = factory();
    showWindowMenu(button[ButtonMenu]->mapToGlobal(p));
    if(!f->exists(this)) return;
    button[ButtonMenu]->setDown(false);
    }
}


#include "lightclient.moc"
