/*******************************************************************************
 *                               lightclient.h                                 *
 *              =---------------------------------------------=                *
 *                begin       : Jan 06 2006                                    *
 *                author      : Jun Oyama (aka Metrocrusader)  (C) 2006        *
 *                email       : metrocrusader@mail.com                         *
 *******************************************************************************/
/*
 *      Copyright  :     See COPYING file that comes with this distribution
 */




#ifndef LIGHTCLIENT_H
#define LIGHTCLIENT_H

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>



class QSpacerItem;
class QPoint;
class QImage;
class QPixmap;
class QTimer;


namespace Light {

class LightClient;

static const int lightEffect_steps = 13; // global constant

enum ButtonType {
  ButtonHelp = 0,
  ButtonMax,
  ButtonMin,
  ButtonClose,
  ButtonMenu,
  ButtonSticky,
  ButtonAbove,
  ButtonBelow,
  ButtonShade,
  ButtonTypeCount
};


class LightFactory : public KDecorationFactory
{

public:
  LightFactory();
  virtual ~LightFactory();
  virtual KDecoration *createDecoration(KDecorationBridge *b);
  virtual bool reset(unsigned long changed);

  static bool initialized();

  static Qt::AlignmentFlags titleAlign();
  static int timerInterval();
  static int titleSize();
  static int frameSize();
  static uchar frameLocationFlag();
  static int gradientType();
  static int brightness();
  static int Spacing();
  static QRgb buttonColor(int);
  static bool defaultBright();
  static bool roundCorner();

  static QPixmap *pixmapCacheTable(int);
  static QImage *buttonImageCache();
  
  void createAlphaMask(QImage *,const QImage *, const int Alpha) const;


private:
  bool readConfig();
  
private:
  static bool flag_initialized;
  static QPixmap *pixmapCacheTable_[lightEffect_steps];
  static QImage *buttonImageCache_;

  // configuration stuff
  static Qt::AlignmentFlags titleAlign_;
  static int timerInterval_;
  static int titleSize_;
  static int frameSize_;
  static uchar frameLocationFlag_; 
  static int gradientType_;
  static int brightness_;
  static int Spacing_;
  static QRgb buttonColor_[ButtonTypeCount];
  static bool defaultBright_;
  static bool roundCorner_;
  
};


inline bool LightFactory::initialized()
{
  return flag_initialized; 
}

inline Qt::AlignmentFlags LightFactory::titleAlign()
{
  return titleAlign_;
}

inline int LightFactory::timerInterval()
{
  return timerInterval_;
}

inline int LightFactory::titleSize()
{
  return titleSize_;
}

inline int LightFactory::frameSize()
{
  return frameSize_;
}

inline uchar LightFactory::frameLocationFlag()
{
  return frameLocationFlag_;
}

inline int LightFactory::gradientType()
{
  return gradientType_;
}

inline int LightFactory::brightness()
{
  return brightness_;
}

inline int LightFactory::Spacing()
{
  return Spacing_;
}  

inline QRgb LightFactory::buttonColor(int type)
{
  return buttonColor_[type];
}

inline bool LightFactory::defaultBright()
{
  return defaultBright_;
}

inline QPixmap *LightFactory::pixmapCacheTable(int step)
{
  return pixmapCacheTable_[step];
}

inline QImage *LightFactory::buttonImageCache() 
{ 
  return buttonImageCache_; 
}

inline bool LightFactory::roundCorner()
{
  return roundCorner_;
}



class LightButton : public QButton
{
  Q_OBJECT

public:
  LightButton(LightClient *parent = 0, const char *name = 0,
	      const QString &tip=NULL,
	      ButtonType buttontype=ButtonHelp);
  ~LightButton();
  

  QSize sizeHint() const; 
  ButtonState lastMousePress() const;
  
public slots:
  void reset();              // because parent's redraw forces button redraw also.
  void timerDone(); 

protected:
  void colorizeButton(QImage *,const QImage *, const QRgb) const; 



  void paintEvent(QPaintEvent *);
  void enterEvent(QEvent *);
  void leaveEvent(QEvent *);
  void mousePressEvent(QMouseEvent *);
  void mouseReleaseEvent(QMouseEvent *);
  
private:
  QSize       size;
  QPixmap     *pixmap_btn;
  LightClient *client;       // its parent
  ButtonType  type;
  ButtonState lastmouse;
  bool        flag_insideButton;
  int         currentStep;
  QTimer      *lightTimer;
};

inline Qt::ButtonState LightButton::lastMousePress() const
{
  return lastmouse;
}

inline void LightButton::reset()
{
  repaint(false);
}



class LightClient : public KDecoration
{
  Q_OBJECT

signals:
  void buttonRepaint();               // connected to each button's reset

public:
  LightClient(KDecorationBridge *b,KDecorationFactory *f);
  virtual ~LightClient();
  
  virtual void init(); 
  virtual void borders(int &,int &,int &,int &) const;
  virtual void resize(const QSize &);
  virtual QSize minimumSize() const;
  
  void resizeEvent(QResizeEvent *);
  void paintEvent(QPaintEvent *);
  void showEvent(QShowEvent *);
  void mouseDoubleClickEvent(QMouseEvent *);

  virtual void maximizeChange();
  virtual void activeChange();
  virtual void iconChange();
  virtual void desktopChange();
  virtual void shadeChange();
  virtual void captionChange();

  void doShape();

  virtual KDecoration::Position mousePosition(const QPoint &) const;
  bool eventFilter(QObject *,QEvent *);

  QPixmap titlebar_bg;                    // for background   
  
private:
  void addButtons(QBoxLayout *layout,const QString &buttons);

private slots:  
  void maxButtonPressed();
  void menuButtonPressed();
  void aboveButtonPressed();  
  void belowButtonPressed();
  void shadeButtonPressed();
  void keepAboveChange(bool);
  void keepBelowChange(bool);


private:
  LightButton *button[ButtonTypeCount];
  QHBoxLayout *entire_titlebar;
  QSpacerItem *titlebar;
};

} // namespace


#endif // LIGHTCLIENT_H
